"""honeybee-display commands."""
import click
import sys
import os
import logging
import json
import pickle

from honeybee.model import Model
from honeybee.cli import main

from honeybee_display.attr import FaceAttribute, RoomAttribute

_logger = logging.getLogger(__name__)


# command group for all display extension commands.
@click.group(help='honeybee display commands.')
@click.version_option()
def display():
    pass


@display.command('model-to-vis')
@click.argument('model-file', type=click.Path(
    exists=True, file_okay=True, dir_okay=False, resolve_path=True))
@click.option(
    '--color-by', '-c', help='Text for the property that dictates the colors of '
    'the Model geometry. Choose from: type, boundary_condition, none. '
    'If none, only a wireframe of the Model will be generated (assuming the '
    '--exclude-wireframe option is not used). None is useful when the primary '
    'purpose of  the visualization is to display results in relation to the Model '
    'geometry or display some room_attr or face_attr as an AnalysisGeometry '
    'or Text labels.', type=str, default='type', show_default=True)
@click.option(
    '--wireframe/--exclude-wireframe', ' /-xw', help='Flag to note whether a '
    'ContextGeometry dedicated to the Model Wireframe (in DisplayLineSegment3D) should '
    'be included in the output VisualizationSet.', default=True, show_default=True)
@click.option(
    '--mesh/--faces', help='Flag to note whether the colored model geometries should '
    'be represented with DisplayMesh3D objects instead of DisplayFace3D objects. '
    'Meshes can usually be rendered  faster and they scale well for large models '
    'but all geometry is triangulated (meaning that their wireframe in certain '
    'platforms might not appear ideal).', default=True, show_default=True)
@click.option(
    '--show-color-by/--hide-color-by', ' /-hcb', help='Flag to note whether the '
    'color-by geometry should be hidden or shown by default. Hiding the color-by '
    'geometry is useful when the primary purpose of the visualization is to display '
    'grid-data or room/face attributes but it is still desirable to have the option '
    'to turn on the geometry.', default=True, show_default=True)
@click.option(
    '--room-attr', '-r', help='An optional text string of an attribute that the Model '
    'Rooms have, which will be used to construct a visualization of this attribute '
    'in the resulting VisualizationSet. Multiple instances of this option can be passed '
    'and a separate VisualizationData will be added to the AnalysisGeometry that '
    'represents the attribute in the resulting VisualizationSet (or a separate '
    'ContextGeometry layer if room_text_labels is True). Room attributes '
    'input here can have . that separates the nested attributes from '
    'one another. For example, properties.energy.program_type.',
    type=click.STRING, multiple=True, default=None, show_default=True)
@click.option(
    '--face-attr', '-f', help='An optional text string of an attribute that the Model '
    'Faces have, which will be used to construct a visualization of this attribute in '
    'the resulting VisualizationSet. Multiple instances of this option can be passed and'
    ' a separate VisualizationData will be added to the AnalysisGeometry that '
    'represents the attribute in the resulting VisualizationSet (or a separate '
    'ContextGeometry layer if face_text_labels is True). Face attributes '
    'input here can have . that separates the nested attributes from '
    'one another. For example, properties.energy.construction.',
    type=click.STRING, multiple=True, default=None, show_default=True)
@click.option(
    '--color-attr/--text-attr', help='Flag to note whether to note whether the '
    'input room-attr and face-attr should be expressed as a colored AnalysisGeometry '
    'or a ContextGeometry as text labels.', default=True, show_default=True)
@click.option(
    '--grid-display-mode', '-m', help='Text that dictates how the ContextGeometry '
    'for Model SensorGrids should display in the resulting visualization. The Default '
    'option will draw sensor points whenever there is no grid_data_path and will not '
    'draw them at all when grid data is provided, assuming the AnalysisGeometry of '
    'the grids is sufficient. Choose from: Default, Points, Wireframe, Surface, '
    'SurfaceWithEdges, None.',
    type=str, default='Default', show_default=True)
@click.option(
    '--hide-grid/--show-grid', ' /-sg', help='Flag to note whether the SensorGrid '
    'ContextGeometry should be hidden or shown by default.',
    default=True, show_default=True)
@click.option(
    '--grid-data', '-g', help='An optional path to a folder containing data that '
    'aligns with the SensorGrids in the model. Any sub folder within this path '
    'that contains a grids_into.json (and associated CSV files) will be '
    'converted to an AnalysisGeometry in the resulting VisualizationSet. '
    'If a vis_metadata.json file is found within this sub-folder, the '
    'information contained within it will be used to customize the '
    'AnalysisGeometry. Note that it is acceptable if data and '
    'grids_info.json exist in the root of this grid_data_path. Also '
    'note that this argument has no impact if honeybee-radiance is not '
    'installed and SensorGrids cannot be decoded.',
    default=None, show_default=True,
    type=click.Path(file_okay=False, dir_okay=True, resolve_path=True))
@click.option(
    '--grid-data-display-mode', '-dm', help='Text to set the display_mode of the '
    'AnalysisGeometry that is generated from the grid_data_path above. Note '
    'that this has no effect if there are no meshes associated with the model '
    'SensorGrids. Choose from: Surface, SurfaceWithEdges, Wireframe, Points',
    type=str, default='Surface', show_default=True)
@click.option(
    '--active-grid-data', '-ad', help='Text to specify the active data in the '
    'AnalysisGeometry. This should match the name of the sub-folder '
    'within the grid_data_path that should be active. If unspecified, the '
    'first data set in the grid-data with be active.',
    type=str, default=None, show_default=True)
@click.option(
    '--output-format', '-of', help='Text for the output format of the resulting '
    'VisualizationSet File (.vsf). Choose from: vsf, json, pkl, vtkjs, html. Note '
    'that both vsf and json refer to the the JSON version of the VisualizationSet '
    'file and the distinction between the two is only for help in coordinating file '
    'extensions (since both .vsf and .json can be acceptable). Also note that '
    'ladybug-vtk must be installed in order for the vtkjs or html options to be usable '
    'and the html format refers to a web page with the vtkjs file embedded within it. '
    'The vtkjs and html options also require an explicit --output-file to be specified.',
    type=str, default='vsf', show_default=True)
@click.option(
    '--output-file', help='Optional file to output the JSON string of '
    'the config object. By default, it will be printed out to stdout',
    type=click.File('w'), default='-', show_default=True)
def model_to_vis_set(
        model_file, color_by, wireframe, mesh, show_color_by,
        room_attr, face_attr, color_attr, grid_display_mode, hide_grid,
        grid_data, grid_data_display_mode, active_grid_data, output_format, output_file):
    """Translate a Honeybee Model file (.hbjson) to a VisualizationSet file (.vsf).

    This command can also optionally translate the Honeybee Model to a .vtkjs file,
    which can be visualized in the open source Visual ToolKit (VTK) platform.

    \b
    Args:
        model_file: Full path to a Honeybee Model (HBJSON or HBpkl) file.
    """
    try:
        model_obj = Model.from_file(model_file)
        room_attrs = [] if len(room_attr) == 0 or room_attr[0] == '' else room_attr
        face_attrs = [] if len(face_attr) == 0 or face_attr[0] == '' else face_attr
        text_labels = not color_attr
        hide_color_by = not show_color_by

        face_attributes = []
        for fa in face_attrs:
            faa = FaceAttribute(name=fa, attrs=[fa], color=color_attr, text=text_labels)
            face_attributes.append(faa)

        room_attributes = []
        for ra in room_attrs:
            raa = RoomAttribute(name=ra, attrs=[ra], color=color_attr, text=text_labels)
            room_attributes.append(raa)

        vis_set = model_obj.to_vis_set(
            color_by=color_by, include_wireframe=wireframe, use_mesh=mesh,
            hide_color_by=hide_color_by, room_attrs=room_attributes,
            face_attrs=face_attributes, grid_display_mode=grid_display_mode,
            hide_grid=hide_grid, grid_data_path=grid_data,
            grid_data_display_mode=grid_data_display_mode,
            active_grid_data=active_grid_data)
        output_format = output_format.lower()
        if output_format in ('vsf', 'json'):
            output_file.write(json.dumps(vis_set.to_dict()))
        elif output_format == 'pkl':
            if output_file.name != '<stdout>':
                out_folder, out_file = os.path.split(output_file.name)
                vis_set.to_pkl(out_file, out_folder)
            else:
                output_file.write(pickle.dumps(vis_set.to_dict()))
        elif output_format in ('vtkjs', 'html'):
            assert output_file.name != '<stdout>', \
                'Must specify an --output-file to use --output-format vtkjs.'
            out_folder, out_file = os.path.split(output_file.name)
            try:
                if out_file.endswith('.vtkjs'):
                    out_file = out_file[:-6]
                elif out_file.endswith('.html'):
                    out_file = out_file[:-5]
                if output_format == 'vtkjs':
                    vis_set.to_vtkjs(output_folder=out_folder, file_name=out_file)
                if output_format == 'html':
                    vis_set.to_html(output_folder=out_folder, file_name=out_file)
            except AttributeError as ae:
                raise AttributeError(
                    'Ladybug-vtk must be installed in order to use --output-format '
                    'vtkjs.\n{}'.format(ae))
        else:
            raise ValueError('Unrecognized output-format "{}".'.format(output_format))
    except Exception as e:
        _logger.exception('Failed to translate Model to VisualizationSet.\n{}'.format(e))
        sys.exit(1)
    else:
        sys.exit(0)


# add display sub-group to honeybee CLI
main.add_command(display)



"""Method to translate a Color Room/Face objects to a VisualizationSet."""
import math

from ladybug_geometry.geometry3d import Vector3D, Point3D, Polyline3D, Plane, \
    Face3D, Polyface3D
from ladybug_display.geometry3d import DisplayText3D
from ladybug_display.visualization import VisualizationSet, ContextGeometry, \
    AnalysisGeometry, VisualizationData
from honeybee.units import conversion_factor_to_meters, parse_distance_string
from honeybee.facetype import Floor

from ..colorobj import _room_wireframe, _process_wireframe


def energy_color_room_to_vis_set(
        color_room, include_wireframe=True, text_labels=False,
        units=None, tolerance=0.01):
    """Translate a Honeybee-Energy ColorRoom to a VisualizationSet.

    Args:
        color_room: A Honeybee-Energy ColorRoom object to be converted to a
            VisualizationSet.
        include_wireframe: Boolean to note whether a ContextGeometry just for
            the Wireframe (in LineSegment3D) should be included. (Default: True).
        text_labels: A boolean to note whether the results should be expressed
            as a colored AnalysisGeometry (False) or a ContextGeometry as text
            labels (True). (Default: False).
        units: Optional text, which will be used to set the default maximum text
            height and the distance of the text to the ground. If None, some
            generic defaults will be used. (Default: None).
        tolerance: Optional tolerance value, which is used to compute the text
            label point for concave geometries. (Default: 0.01).

    Returns:
        A VisualizationSet object that represents the ColorRoom with an
        AnalysisGeometry when text_labels is False or a ContextGeometry when
        text_labels is True. It can also optionally include a ContextGeometry
        for the wireframe.
    """
    # set up an empty visualization set
    vis_set = VisualizationSet(color_room.data_type_text.replace(' ', '_'), ())
    vis_set.display_name = color_room.data_type_text

    # use text labels if requested
    if text_labels:
        txt_height, font, f_str = _process_leg_par_for_text(color_room)
        # loop through the rooms and create the text labels
        max_txt_h, p_tol = float('inf'), 0.01
        if units is not None:
            fac_to_m = conversion_factor_to_meters(units)
            max_txt_h = 0.25 / fac_to_m
            max_txt_v = 1.0 / fac_to_m
            p_tol = parse_distance_string('0.01m', units)
        label_text = []
        for room_val, room in zip(color_room.matched_values, color_room.matched_rooms):
            room_prop = f_str % room_val
            # compute the center point for the text
            if units is not None:
                room_h = room.geometry.max.z - room.geometry.min.z
                m_vec = Vector3D(0, 0, max_txt_v) if room_h > max_txt_v * 2 \
                    else Vector3D(0, 0, room_h / 2)
                floor_faces = [face.geometry for face in room.faces
                               if isinstance(face.type, Floor)]
                if len(floor_faces) == 1:
                    flr_geo = floor_faces[0]
                    base_pt = flr_geo.center if flr_geo.is_convex else \
                        flr_geo.pole_of_inaccessibility(p_tol)
                elif len(floor_faces) == 0:
                    c_pt = room.geometry.center
                    base_pt = Point3D(c_pt.x, c_pt.y, room.geometry.min.z)
                else:
                    floor_p_face = Polyface3D.from_faces(floor_faces, tolerance)
                    ne = floor_p_face.naked_edges
                    floor_outline = Polyline3D.join_segments(ne, tolerance)[0]
                    flr_geo = Face3D(floor_outline.vertices[:-1])
                    base_pt = flr_geo.center if flr_geo.is_convex else \
                        flr_geo.pole_of_inaccessibility(p_tol)
                base_pt = base_pt.move(m_vec)
                base_plane = Plane(Vector3D(0, 0, 1), base_pt)
            else:
                base_pt = room.geometry.center
            base_plane = Plane(Vector3D(0, 0, 1), base_pt)
            # get the text height
            if txt_height is None:  # auto-calculate default text height
                txt_len = len(room_prop) if len(room_prop) > 10 else 10
                txt_h = (room.geometry.max.x - room.geometry.min.x) / txt_len
            else:
                txt_h = txt_height
            txt_h = max_txt_h if txt_h > max_txt_h else txt_h
            # create the text label
            label = DisplayText3D(
                room_prop, base_plane, txt_h, font=font,
                horizontal_alignment='Center', vertical_alignment='Middle')
            label_text.append(label)  # append everything to the list
        con_geo = ContextGeometry(vis_set.identifier, label_text)
        con_geo.display_name = vis_set.display_name
        vis_set.add_geometry(con_geo)
    else:  # use a colored AnalysisGeometry
        # create the analysis geometry
        vis_data = VisualizationData(
            color_room.matched_values, color_room.legend_parameters,
            color_room.data_type, str(color_room.unit))
        geo = tuple(room.geometry for room in color_room.matched_rooms)
        a_geo = AnalysisGeometry(vis_set.identifier, geo, [vis_data])
        a_geo.display_name = vis_set.display_name
        vis_set.add_geometry(a_geo)

    # loop through all of the rooms and add their wire frames
    if include_wireframe:
        vis_set.add_geometry(_room_wireframe(color_room.rooms))
    return vis_set


def color_face_to_vis_set(
        color_face, include_wireframe=True, text_labels=False,
        units=None, tolerance=0.01):
    """Translate a Honeybee ColorFace to a VisualizationSet.

    Args:
        color_face: A Honeybee ColorFace object to be converted to a VisualizationSet.
        include_wireframe: Boolean to note whether a ContextGeometry just for
            the Wireframe (in LineSegment3D) should be included. (Default: True).
        text_labels: A boolean to note whether the attribute assigned to the
            ColorFace should be expressed as a colored AnalysisGeometry (False)
            or a ContextGeometry as text labels (True). (Default: False).
        units: Optional text, which will be used to set the default maximum text
            height and the distance of the text to the ground. If None, some
            generic defaults will be used. (Default: None).
        tolerance: Tolerance value, which is used to eliminate very small
            text. (Default: 0.01).

    Returns:
        A VisualizationSet object that represents the ColorFace with an
        AnalysisGeometry when text_labels is False or a ContextGeometry when
        text_labels is True. It can also optionally include a ContextGeometry
        for the wireframe.
    """
    # set up an empty visualization set
    vis_set = VisualizationSet(color_face.data_type_text.replace(' ', '_'), ())
    vis_set.display_name = color_face.data_type_text

    # use text labels if requested
    if text_labels:
        # set up default variables
        max_txt_h, p_tol = float('inf'), 0.01
        if units is not None:
            fac_to_m = conversion_factor_to_meters(units)
            max_txt_h = 0.25 / fac_to_m
            p_tol = parse_distance_string('0.01m', units)
        txt_height, font, f_str = _process_leg_par_for_text(color_face)
        # loop through the faces and create the text labels
        label_text = []
        face_zip_obj = zip(color_face.matched_values, color_face.matched_flat_geometry)
        for face_val, f_geo in face_zip_obj:
            cent_pt = f_geo.center if f_geo.is_convex else \
                f_geo.pole_of_inaccessibility(p_tol)
            base_plane = Plane(f_geo.normal, cent_pt)
            face_prop = f_str % face_val
            if base_plane.y.z < 0:  # base plane pointing downwards; rotate it
                base_plane = base_plane.rotate(base_plane.n, math.pi, base_plane.o)
            if txt_height is None:  # auto-calculate default text height
                txt_len = len(face_prop) if len(face_prop) > 10 else 10
                dims = [
                    (f_geo.max.x - f_geo.min.x),
                    (f_geo.max.y - f_geo.min.y),
                    (f_geo.max.z - f_geo.min.z)]
                dims.sort()
                txt_h = dims[1] / txt_len
            else:
                txt_h = txt_height
            if txt_h < tolerance:
                continue
            txt_h = max_txt_h if txt_h > max_txt_h else txt_h
            # move base plane origin a little to avoid overlaps of adjacent labels
            if base_plane.n.x != 0:
                m_vec = base_plane.y if base_plane.n.x < 0 else -base_plane.y
            else:
                m_vec = base_plane.y if base_plane.n.z < 0 else -base_plane.y
            base_plane = base_plane.move(m_vec * txt_h)
            # create the text label
            label = DisplayText3D(
                face_prop, base_plane, txt_h, font=font,
                horizontal_alignment='Center', vertical_alignment='Middle')
            label_text.append(label)  # append everything to the list
        con_geo = ContextGeometry(vis_set.identifier, label_text)
        con_geo.display_name = vis_set.display_name
        vis_set.add_geometry(con_geo)
    else:  # use a colored AnalysisGeometry
        # create the analysis geometry
        vis_data = VisualizationData(
            color_face.matched_values, color_face.legend_parameters,
            color_face.data_type, str(color_face.unit))
        a_geo = AnalysisGeometry(
            vis_set.identifier, color_face.matched_flat_geometry, [vis_data])
        a_geo.display_name = vis_set.display_name
        vis_set.add_geometry(a_geo)

    # loop through all of the rooms and add their wire frames
    if include_wireframe:
        wireframe = []
        for face in color_face.faces:
            _process_wireframe(face, wireframe, 2)
            for ap in face._apertures:
                _process_wireframe(ap, wireframe)
            for dr in face._doors:
                _process_wireframe(dr, wireframe)
        con_geo = ContextGeometry('Wireframe', wireframe)
        vis_set.add_geometry(con_geo)
    return vis_set


def _process_leg_par_for_text(color_obj):
    """Get the relevant Legend Parameters for DisplayText3D."""
    txt_height = None if color_obj.legend_parameters.is_text_height_default \
        else color_obj.legend_parameters.text_height
    font = color_obj.legend_parameters.font
    f_str = '%.{}f'.format(color_obj.legend_parameters.decimal_count)
    return txt_height, font, f_str



"""Display attribute objects for creating visualization sets."""
from honeybee.facetype import Wall, RoofCeiling, Floor, AirBoundary
from honeybee.aperture import Aperture
from honeybee.shade import Shade
from honeybee.boundarycondition import Outdoors, Surface, Ground


class RoomAttribute(object):
    """A Room attribute object.

    Args:
        name: A name for this Room Attribute.
        attrs: A list of text strings of attributes that the Model Rooms have, which will
            be used to construct a visualization of this attribute in the resulting
            VisualizationSet. This can also be a list of attribute strings and a
            separate VisualizationData will be added to the AnalysisGeometry that
            represents the attribute in the resulting VisualizationSet (or a separate
            ContextGeometry layer if color is True). Attributes input here can have '.'
            that separates the nested attributes from one another. For example,
            'properties.energy.construction' or 'user_data.tag'
        color: A boolean to note whether the input room_attr should be expressed as a
            colored AnalysisGeometry. (Default: True)
        text: A boolean to note whether the input room_attr should be expressed as a
            a ContextGeometry as text labels. (Default: False)
        legend_par:An optional LegendParameter object to customize the display of the
            attribute. For text attribute only the text_height and font will be used to
            customize the text.
    """

    def __init__(
        self, name, attrs, color=True, text=False, legend_par=None
            ):
        self.name = name
        self.attrs = attrs
        self.color = color
        self.text = text
        self.legend_par = legend_par

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def attrs(self):
        return self._attrs

    @attrs.setter
    def attrs(self, value):
        self._attrs = value

    @property
    def color(self):
        return self._color

    @color.setter
    def color(self, value):
        self._color = value

    @property
    def text(self):
        return self._text

    @text.setter
    def text(self, value):
        self._text = value

    @property
    def legend_par(self):
        return self._legend_par

    @legend_par.setter
    def legend_par(self, value):
        self._legend_par = value


class FaceAttribute(RoomAttribute):
    """A Face attribute object.

    Args:
        name: A name for this Face Attribute.
        attrs: A list of text strings of attributes the Model Faces have, which will
            be used to construct a visualization of this attribute in the resulting
            VisualizationSet. This can also be a list of attribute strings and a
            separate VisualizationData will be added to the AnalysisGeometry that
            represents the attribute in the resulting VisualizationSet (or a separate
            ContextGeometry layer if color is True). Attributes input here can have '.'
            that separates the nested attributes from one another. For example,
            'properties.energy.construction' or 'user_data.tag'

        color: A boolean to note whether the input room_attr should be expressed as a
            colored AnalysisGeometry. (Default: True)

        text: A boolean to note whether the input room_attr should be expressed as a
            a ContextGeometry as text labels. (Default: False)

        legend_par:An optional LegendParameter object to customize the display of the
            attribute. For text attribute only the text_height and font will be used to
            customize the text.

        face_types: List of face types to be included in the visualization set. By
            default all the faces will be exported to visualization set. Valid values
            are:

            * Wall
            * RoofCeiling
            * Floor
            * AirBoundary
            * Aperture
            * Shade

        boundary_conditions: List of face boundary conditions to be included in the
            visualization set. This condition will be applied as a secondary check for
            the face_types that are set using the face_types argument. Valid values 
            are:

            * Outdoors
            * Surface
            * Ground

    """

    def __init__(
        self, name, attrs, color=True, text=False, legend_par=None, face_types=None,
            boundary_conditions=None):
        super().__init__(name, attrs, color, text, legend_par)
        self.face_types = face_types
        self.boundary_conditions = boundary_conditions

    @property
    def face_types(self):
        return self._face_types

    @face_types.setter
    def face_types(self, types):
        types = types or []
        for type in types:
            assert type in (Wall, RoofCeiling, Floor, AirBoundary, Aperture, Shade), \
                f'Invalid face type: {type}'
        self._face_types = types

    @property
    def boundary_conditions(self):
        return self._boundary_conditions

    @boundary_conditions.setter
    def boundary_conditions(self, bcs):
        bcs = bcs or []
        for bc in bcs:
            assert bc in (Outdoors, Surface, Ground), \
                f'Invalid face boundary condition: {bc}. Valid values are Outdoors, ' \
                'Surface and Ground.'
        self._boundary_conditions = bcs



"""Test the Model to_vis_set method."""
from ladybug_display.geometry3d import DisplayMesh3D, DisplayLineSegment3D, \
    DisplayText3D
from ladybug_display.visualization import VisualizationSet, \
    ContextGeometry, AnalysisGeometry, VisualizationData
from honeybee.model import Model
from honeybee_display.attr import RoomAttribute, FaceAttribute


def test_default_to_vis_set():
    """Test the default output of Model.to_vis_set()."""
    model_json = './tests/json/single_family_home.hbjson'
    parsed_model = Model.from_hbjson(model_json)
    vis_set = parsed_model.to_vis_set()

    assert isinstance(vis_set, VisualizationSet)
    assert len(vis_set) == 7
    for geo_obj in vis_set[:-1]:
        assert isinstance(geo_obj, ContextGeometry)
        assert isinstance(geo_obj[0], DisplayMesh3D)
    assert isinstance(vis_set[-1], ContextGeometry)
    assert vis_set[-1].display_name == 'Wireframe'
    assert isinstance(vis_set[-1][0], DisplayLineSegment3D)

    vis_set = parsed_model.to_vis_set(include_wireframe=False)
    assert len(vis_set) == 6
    for geo_obj in vis_set:
        assert isinstance(geo_obj, ContextGeometry)
        assert isinstance(geo_obj[0], DisplayMesh3D)

    vis_set = parsed_model.to_vis_set('none')
    assert len(vis_set) == 1
    for geo_obj in vis_set:
        assert isinstance(geo_obj, ContextGeometry)
        assert isinstance(geo_obj[0], DisplayLineSegment3D)

    vis_set = parsed_model.to_vis_set('boundary_condition', include_wireframe=False)

    assert len(vis_set) == 4
    for geo_obj in vis_set:
        assert isinstance(geo_obj, ContextGeometry)
        assert isinstance(geo_obj[0], DisplayMesh3D)

    vis_set = parsed_model.to_vis_set_wireframe()
    assert len(vis_set) == 1
    for geo_obj in vis_set:
        assert isinstance(geo_obj, ContextGeometry)
        assert isinstance(geo_obj[0], DisplayLineSegment3D)


def test_room_attr_to_vis_set():
    """Test the room attribute argument of Model.to_vis_set()."""
    model_json = './tests/json/single_family_home.hbjson'
    parsed_model = Model.from_hbjson(model_json)
    attr_color = RoomAttribute(
        name='Floor Area', attrs=['floor_area'], text=False, color=True)
    vis_set = parsed_model.to_vis_set('none', room_attrs=[attr_color])

    assert isinstance(vis_set[0], AnalysisGeometry)
    assert isinstance(vis_set[0][0], VisualizationData)
    assert len(vis_set[0][0].values) == 7
    attr_txt = RoomAttribute(
        name='Floor Area', attrs=['floor_area'], text=True, color=False)
    vis_set = parsed_model.to_vis_set('none', room_attrs=[attr_txt])
    assert isinstance(vis_set[0], ContextGeometry)
    assert len(vis_set[0]) == 7
    for item in vis_set[0]:
        assert isinstance(item, DisplayText3D)


def test_face_attr_to_vis_set():
    """Test the face attribute argument of Model.to_vis_set()."""
    model_json = './tests/json/single_family_home.hbjson'
    parsed_model = Model.from_hbjson(model_json)
    attr_color = FaceAttribute(name='Area', attrs=['area'], color=True, text=False)
    vis_set = parsed_model.to_vis_set('None', face_attrs=[attr_color])
    assert isinstance(vis_set[0], AnalysisGeometry)
    assert isinstance(vis_set[0][0], VisualizationData)
    assert len(vis_set[0][0].values) == 140

    attr_txt = FaceAttribute(name='Area', attrs=['area'], color=False, text=True)
    vis_set = parsed_model.to_vis_set('None', face_attrs=[attr_txt])
    assert isinstance(vis_set[0], ContextGeometry)
    assert len(vis_set[0]) == 140
    for item in vis_set[0]:
        assert isinstance(item, DisplayText3D)



"""Test the Shade to_vis_set method."""
from ladybug_geometry.geometry3d import Point3D, Face3D
from honeybee.shade import Shade
from ladybug_display.geometry3d import DisplayFace3D
from ladybug_display.visualization import VisualizationSet, ContextGeometry


def test_shade_to_vis_set():
    """Test the default output of Shade.to_vis_set()."""
    pts = (Point3D(0, 0, 0), Point3D(0, 0, 3), Point3D(1, 0, 3), Point3D(1, 0, 0))
    shade = Shade('TestShade', Face3D(pts))
    vis_set = shade.to_vis_set()

    assert isinstance(vis_set, VisualizationSet)
    assert len(vis_set) == 1
    assert isinstance(vis_set[0], ContextGeometry)
    for geo in vis_set[0]:
        assert isinstance(geo, DisplayFace3D)
